@@ -186,6 +186,7 @@ pub struct RuleParser {
 }
 
 /// Errors from the CSS parsing process
+#[allow(dead_code)] // looks like we are not actually using this yet?
 #[derive(Debug)]
 pub enum ParseErrorKind<'i> {
     Selector(selectors::parser::SelectorParseErrorKind<'i>),
@@ -697,7 +698,11 @@ where {
                 .xml_lang()
                 .0
                 .as_ref()
-                .map_or(false, |e_lang| css_lang.iter().any(|l| l.matches(e_lang))),
+                .map_or(false, |e_lang| {
+                    css_lang
+                        .iter()
+                        .any(|l| l.is_language_range() && l.matches(e_lang))
+                }),
         }
     }
 
