@@ -24,8 +24,9 @@ use std::f64;
 use std::f64::consts::*;
 use std::slice;
 
-use crate::float_eq_cairo::ApproxEqCairo;
+use crate::float_eq_cairo::{ApproxEqCairo, CAIRO_FIXED_MAX_DOUBLE, CAIRO_FIXED_MIN_DOUBLE};
 use crate::path_parser::{ParseError, PathParser};
+use crate::transform::Transform;
 use crate::util::clamp;
 
 /// Whether an arc's sweep should be >= 180 degrees, or smaller.
@@ -420,6 +421,37 @@ impl PathCommand {
             )),
         }
     }
+
+    /// Sees if any of the coordinates in the path is not representable in Cairo's fixed-point numbers.
+    ///
+    /// See the documentation for [`Path::has_unsuitable_coordinates`].
+    fn has_unsuitable_coordinates(&self, transform: &Transform) -> bool {
+        match *self {
+            PathCommand::MoveTo(x, y) => coordinates_are_unsuitable(x, y, transform),
+            PathCommand::LineTo(x, y) => coordinates_are_unsuitable(x, y, transform),
+            PathCommand::CurveTo(CubicBezierCurve { pt1, pt2, to }) => {
+                coordinates_are_unsuitable(pt1.0, pt1.1, transform)
+                    || coordinates_are_unsuitable(pt2.0, pt2.1, transform)
+                    || coordinates_are_unsuitable(to.0, to.1, transform)
+            }
+
+            PathCommand::Arc(EllipticalArc { r, from, to, .. }) => {
+                coordinates_are_unsuitable(r.0, r.1, transform)
+                    || coordinates_are_unsuitable(from.0, from.1, transform)
+                    || coordinates_are_unsuitable(to.0, to.1, transform)
+            }
+
+            PathCommand::ClosePath => false,
+        }
+    }
+}
+
+fn coordinates_are_unsuitable(x: f64, y: f64, transform: &Transform) -> bool {
+    let fixed_point_range = CAIRO_FIXED_MIN_DOUBLE..=CAIRO_FIXED_MAX_DOUBLE;
+
+    let (x, y) = transform.transform_point(x, y);
+
+    !(fixed_point_range.contains(&x) && fixed_point_range.contains(&y))
 }
 
 /// Constructs a path out of commands.
@@ -720,6 +752,26 @@ impl Path {
     pub fn is_empty(&self) -> bool {
         self.commands.is_empty()
     }
+
+    /// Sees if any of the coordinates in the path is not representable in Cairo's fixed-point numbers.
+    ///
+    /// See https://gitlab.gnome.org/GNOME/librsvg/-/issues/1088 and
+    /// for the root cause
+    /// https://gitlab.freedesktop.org/cairo/cairo/-/issues/852.
+    ///
+    /// This function does a poor job, but a hopefully serviceable one, of seeing if a path's coordinates
+    /// are prone to causing trouble when passed to Cairo.  The caller of this function takes note of
+    /// that situation and in the end avoids rendering the path altogether.
+    ///
+    /// Cairo has trouble when given path coordinates that are outside of the range it can represent
+    /// in cairo_fixed_t: 24 bits integer part, and 8 bits fractional part.  Coordinates outside
+    /// of ±8 million get clamped.  These, or valid coordinates that are close to the limits,
+    /// subsequently cause integer overflow while Cairo does arithmetic on the path's points.
+    /// Fixing this in Cairo is a long-term project.
+    pub fn has_unsuitable_coordinates(&self, transform: &Transform) -> bool {
+        self.iter()
+            .any(|command| command.has_unsuitable_coordinates(transform))
+    }
 }
 
 fn take_one(iter: &mut slice::Iter<'_, f64>) -> f64 {
@@ -870,4 +922,24 @@ mod tests {
             ]
         );
     }
+
+    #[test]
+    fn detects_suitable_coordinates() {
+        let mut builder = PathBuilder::default();
+        builder.move_to(900000.0, 33.0);
+        builder.line_to(-900000.0, 3.0);
+
+        let path = builder.into_path();
+        assert!(!path.has_unsuitable_coordinates(&Transform::identity()));
+    }
+
+    #[test]
+    fn detects_unsuitable_coordinates() {
+        let mut builder = PathBuilder::default();
+        builder.move_to(9000000.0, 33.0);
+        builder.line_to(-9000000.0, 3.0);
+
+        let path = builder.into_path();
+        assert!(path.has_unsuitable_coordinates(&Transform::identity()));
+    }
 }
